//---------------------------------------------------------------------------
/*
        Simple Wind Plot Generator
        by Fred Houweling
        Free for use, source code available.
        Pubic Domain Software.
        fred@houweling.com.au
        http://www.houweling.com.au/r/programming
*/
#include <vcl.h>
#pragma hdrstop
#include <math.h>
#include <graphics.hpp>
#include "WindPlotCode.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
        InnerCircle=clBlue;
        OuterCircle=clGreen;
        Panel3->Color=InnerCircle;
        Panel4->Color=OuterCircle;
        Panel5->Color=clBlack;
        CopyToTable();
        Draw(Owner, false);
}
//---------------------------------------------------------------------------
struct MonthAvgData {
	char *MonthName;
	double pvwatts;
	double wind;
	double dir;
} mavgd[] =
{
	{"January",	1044,	4.8,	23},
	{"February",	1053,	3.3,	90},
	{"March",	987,	3.9,	68},
	{"April",	895,	4.7,	100},
	{"May",	        812,	2.9,	208},
	{"June",	716,	3.1,	84},
	{"July",	764,	3.9,	113},
	{"August",	870,	2.9,	212},
	{"September",	967,	3.2,	66},
	{"October",	1013,	4.8,	10},
	{"November",	1029,	3.7,	36},
	{"December",	1019,	3,	78},
	{"180Deg",	1019,	3,	180},
	{"270Deg",	1019,	3,	270},
	{"0Deg",	1019,	3,	0}
	// 90 deg already exists in the dataset. (Feb)
};
int elements=12;
void __fastcall TForm1::CopyToTable()
{
        StringGrid1->Cells[0][0]="Month";
        StringGrid1->Cells[1][0]="Wind Strength";
        StringGrid1->Cells[2][0]="Wind Direction";
        for(int i=0;i<elements;i++)
        {
                StringGrid1->Cells[0][i+1]=AnsiString(mavgd[i].MonthName);
                StringGrid1->Cells[1][i+1]=mavgd[i].wind;
                StringGrid1->Cells[2][i+1]=mavgd[i].dir;
        }
}
void __fastcall TForm1::CopyFromTable()
{
        for(int i=0;i<elements;i++)
        {
                //StringGrid1->Cells[0][i+1]=AnsiString(mavgd[i].MonthName);
                mavgd[i].wind=StringGrid1->Cells[1][i+1].ToDouble();
                mavgd[i].dir=StringGrid1->Cells[2][i+1].ToInt();
        }
        CopyToTable();
}

void __fastcall TForm1::Draw(TObject *Sender, int WithSave=false)
{
        double maxwind;
        double avgwind;
        double minwind;
        int maxsize;
        int c_x, c_y;
        int p_x, p_y;
        int r;
        TMetafile *pMetafile = new TMetafile;
        // Fix the metafile height and width to the form's TImage Image1 object's size.
        pMetafile->Height=Image1->Height;
        pMetafile->Width=Image1->Width;
        // Allocate a Canvas area to draw to that ultimately will return to the pMetafile object on object deletion.
        TMetafileCanvas *pCanvas = new TMetafileCanvas(pMetafile, 0);
        Image1->Canvas->Rectangle(0,0,Image1->Width,Image1->Height);
        // Preload the first data element into the minwind value
        // (So the real small value can be found.)
        minwind=mavgd[0].wind;
        // Clear the max and avg wind values.
        maxwind=avgwind=0;
        // Find the min/max and avg values for the dataset.
        for(int i=0;i<elements;i++)
        {
                avgwind+=mavgd[i].wind;
                if(mavgd[i].wind>maxwind)
                {
                        maxwind=mavgd[i].wind;
                }
                if(mavgd[i].wind<minwind)
                {
                        minwind=mavgd[i].wind;
                }
        }

        avgwind/=elements;
        c_x=Image1->Width/2;
        c_y=Image1->Height/2;
        maxsize=Image1->Width;
        if(maxsize>Image1->Height)
        {
                maxsize=Image1->Height;
        }
        // Draw max wind circle, maxwind scaled is maxsize / 2 i.e. r
        r=maxsize/2;
        pCanvas->Brush->Color = InnerCircle;
        pCanvas->Ellipse(c_x-r, c_y-r, c_x+r, c_y+r);

        // Draw the average wind circle
        pCanvas->Brush->Color = OuterCircle;
        r=((double)(maxsize/2)/maxwind*avgwind);
        pCanvas->Ellipse(c_x-r, c_y-r, c_x+r, c_y+r);

        // Draw the minimum circle
        pCanvas->Brush->Color = clWhite;
        r=((double)(maxsize/2)/(double)maxwind*(double)minwind);
        pCanvas->Ellipse(c_x-r, c_y-r, c_x+r, c_y+r);
        pCanvas->Pen->Color = Panel5->Color;
        // r is Needed to draw NSEW
        r=((double)(maxsize/2)/maxwind*avgwind);

        // Set the font options
        pCanvas->Brush->Style=bsClear; // Transparent text
        pCanvas->Font->Assign(FontDialog1->Font);
        //pCanvas->Font->Size+=4;
        //pCanvas->Font->Style=TFontStyles()<< fsBold;
        // Draw NSEW
        pCanvas->TextOutA(c_x-pCanvas->TextWidth("N")/2, c_y-(r+pCanvas->TextHeight("N")+4), "N");
        pCanvas->TextOutA(c_x-pCanvas->TextWidth("S")/2, c_y+(r+4), "S");
        pCanvas->TextOutA(c_x+r+4, c_y-pCanvas->TextHeight("E")/2, "E");
        pCanvas->TextOutA(c_x-(r+pCanvas->TextWidth("W")+4), c_y-pCanvas->TextHeight("W")/2, "W");
        // Draw the elements(12 months) wind data.
        for(int i=0;i<elements;i++) // Add 3 to include the 3 test records.
        {
                // To plot the month name uncomment the following line.
                if(CheckBox1->Checked==true)
                {
                        pCanvas->TextOutA(p_x, p_y, AnsiString(mavgd[i].MonthName));
                }
                // Move the beginning of the line.
                pCanvas->MoveTo(c_x, c_y);
                // The following code calculates the location of the x and y location using the degrees,
                // the -90 factor brings the 0 degree back to North instead of East.
                // Find the x and y locations using cos and sin, need to convert degrees to radians
                // Multiply result be result by scaled (To drawing space) wind velocity.
                p_x=c_x+cos(M_PI*2/360*(mavgd[i].dir-90))*((double)(maxsize/2)/maxwind*mavgd[i].wind);
                p_y=c_y+sin(M_PI*2/360*(mavgd[i].dir-90))*((double)(maxsize/2)/maxwind*mavgd[i].wind);
                // Draw the line.
                pCanvas->LineTo(p_x, p_y);
        }
        delete pCanvas; // Deleting the object completes the Metafile creation process.

        Image1->Canvas->Draw(0,0,pMetafile); // Show the image in the Image1 area of the form.
        if(WithSave==true)
        {
                // Save the file
                SaveDialog1->DefaultExt="wmf";
                SaveDialog1->Filter = "Windows Meta File (*.wmf)|*.WMF";
                SaveDialog1->Options.Clear();
                SaveDialog1->Options << ofOverwritePrompt << ofPathMustExist;
                if(SaveDialog1->Execute()==true) // Returns true if a file is selected.
                {
                        pMetafile->SaveToFile(SaveDialog1->FileName); // Save the file specified location.
                }
        }
        delete pMetafile; // Clean up
}

//---------------------------------------------------------------------------
void __fastcall TForm1::Button2Click(TObject *Sender)
{
        if(FontDialog1->Execute()==true)
        {
                Draw(Sender);
        }
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Panel3Click(TObject *Sender)
{
        ColorDialog1->Color=InnerCircle;
        if(ColorDialog1->Execute()==true)
        {
                InnerCircle=ColorDialog1->Color;
                Panel3->Color=InnerCircle;
                Draw(Sender);
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Panel4Click(TObject *Sender)
{
        ColorDialog1->Color=OuterCircle;
        if(ColorDialog1->Execute()==true)
        {
                OuterCircle=ColorDialog1->Color;
                Panel4->Color=OuterCircle;
                Draw(Sender);
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button1Click(TObject *Sender)
{
        Draw(Sender, true);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CheckBox1Click(TObject *Sender)
{
        Draw(Sender, false);        
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Panel5Click(TObject *Sender)
{
        ColorDialog1->Color=Panel5->Color;
        if(ColorDialog1->Execute()==true)
        {
                Panel5->Color=ColorDialog1->Color;
                Draw(Sender);
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::StringGrid1GetEditMask(TObject *Sender, int ACol,
      int ARow, AnsiString &Value)
{
        if(ACol==1)
        {
                Value="999990";
        }
        if(ACol==2)
        {
                Value="990";
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::StringGrid1SetEditText(TObject *Sender, int ACol,
      int ARow, const AnsiString Value)
{
        double tempd;
        int    tempi;
        if(Value.Trim().Length()==0)
        {
                return;
        }
        try
        {
                if(ACol==1)
                {
                        try
                        {
                                tempd=Value.ToDouble();
                                if(tempd>=0)
                                {
                                        mavgd[ARow].wind=tempd;
                                        Draw(Sender, false);
                                }

                        }
                        catch(...)
                        {
                                // Nothing
                        }
                }
                if(ACol==2)
                {
                        try
                        {
                                tempi=Value.ToIntDef(-1);
                                if(tempi>=0&&tempi<=360)
                                {
                                        mavgd[ARow].dir=tempi;
                                        Draw(Sender, false);
                                }
                        }
                        catch(...)
                        {
                                // Nothing
                        }

                }

        }
        catch(...)
        {
                // Do nothing
        }
}
//---------------------------------------------------------------------------


